from typing import Any, Dict, List, Optional, TypedDict


__all__ = [
    "AppDict",
    "AppModel",
    "TableContext",
    "ItemContext",
    "URLConfig",
]


class Perms(TypedDict):
    add: bool
    change: bool
    delete: bool
    view: bool


class AppModel(TypedDict):
    name: str
    object_name: str
    perms: Perms
    admin_url: str
    add_url: Optional[str]
    view_only: bool


class AppDict(TypedDict):
    name: str
    app_label: str
    app_url: str
    has_module_perms: bool
    models: List[AppModel]


class TableContext(TypedDict):
    title: str
    subtitle: Optional[str]
    table: Dict[str, List[Any]]


class SectionData(TypedDict):
    name: Optional[str]
    description: Optional[str]
    fields: Dict[str, Any]


class ItemContext(TypedDict):
    slug: Any
    title: str
    subtitle: Optional[str]
    image: Optional[str]
    data: List[SectionData]


class ItemConfig(TypedDict):
    route: str
    view: str
    name: str


class URLConfig(TypedDict):
    route: str
    view: str
    name: str
    items: Optional[ItemConfig]



from typing import Callable, List, Union

from django.contrib import admin
from django.http import HttpRequest
from django.template.response import TemplateResponse
from django.urls import URLPattern, URLResolver, path

from .settings import admin_data_settings
from .typing import AppDict, AppModel


__all__ = [
    "add_url",
    "get_data_admin_views",
]


def add_url(baseroute: str, route: str, name: str) -> AppModel:
    return {
        "name": " ".join([val.capitalize() for val in name.split("_")]),
        "object_name": name.lower(),
        "perms": {"add": False, "change": False, "delete": False, "view": True},
        "admin_url": f"/admin/{baseroute}/{route}/",
        "add_url": None,
        "view_only": True,
    }


def get_data_admin_views() -> AppDict:
    baseroute = admin_data_settings.NAME.lower().replace(" ", "-")
    return {
        "name": admin_data_settings.NAME,
        "app_label": baseroute,
        "app_url": f"/admin/{baseroute}/",
        "has_module_perms": True,
        "models": [
            add_url(baseroute=baseroute, route=item["route"], name=item["name"]) for item in admin_data_settings.URLS
        ],
    }


# Added to site


def get_app_list(self: admin.AdminSite, request: HttpRequest) -> List[AppDict]:
    baseroute = admin_data_settings.NAME.lower().replace(" ", "-")
    app_dict = self._build_app_dict(request)  # pylint: disable=protected-access
    app_dict[baseroute] = get_data_admin_views()
    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: x["name"].lower())

    # Sort the models alphabetically within each app.
    for app in app_list:
        app["models"].sort(key=lambda x: x["name"])

    return app_list


def admin_data_index_view(self: admin.AdminSite, request: HttpRequest) -> TemplateResponse:
    app_dict = get_data_admin_views()
    # Sort the models alphabetically
    app_dict["models"].sort(key=lambda x: x["name"])

    context = {
        **self.each_context(request),
        "title": app_dict["name"],
        "subtitle": None,
        "app_list": [app_dict],
        "app_label": admin_data_settings.NAME,
    }

    request.current_app = admin_data_settings.NAME
    return TemplateResponse(request, "admin/app_index.html", context)


def get_admin_data_urls(self: admin.AdminSite) -> List[Union[URLResolver, URLPattern]]:
    baseroute = admin_data_settings.NAME.lower().replace(" ", "-")
    custom_paths = [
        path(
            route=f"{baseroute}/",
            view=self.admin_view(self.admin_data_index_view),
            name="admin-data-index-view",
        )
    ]
    for item in admin_data_settings.URLS:
        custom_paths.append(
            path(
                route=f"{baseroute}/{item['route']}/",
                view=self.admin_view(item["view"]),
                name=item["name"],
            )
        )
        if item["items"] is not None:
            custom_paths.append(
                path(
                    route=f"{baseroute}/{item['route']}/{item['items']['route']}/",
                    view=self.admin_view(item["items"]["view"]),
                    name=item["items"]["name"],
                )
            )

    return custom_paths


def get_urls(
    original_get_urls: Callable[[], List[Union[URLResolver, URLPattern]]],
) -> Callable[[admin.AdminSite], List[Union[URLResolver, URLPattern]]]:
    def get_urls_inner(self: admin.AdminSite) -> List[Union[URLResolver, URLPattern]]:
        return self.get_admin_data_urls() + original_get_urls()

    return get_urls_inner


# Patch the admin site object with data admin view methods
# pylint: disable=no-value-for-parameter
admin.site.get_app_list = get_app_list.__get__(admin.site, admin.AdminSite)
admin.site.admin_data_index_view = admin_data_index_view.__get__(admin.site, admin.AdminSite)
admin.site.get_admin_data_urls = get_admin_data_urls.__get__(admin.site, admin.AdminSite)
admin.site.get_urls = get_urls(admin.site.get_urls).__get__(admin.site, admin.AdminSite)



from functools import wraps
from typing import Any, Callable, List

from django.contrib import admin
from django.http import HttpRequest
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.html import format_html

from .settings import admin_data_settings
from .typing import ItemContext, TableContext, URLConfig


__all__ = [
    "render_with_item_view",
    "render_with_table_view",
    "ItemLink",
]


def render_with_table_view(
    func: Callable[..., TableContext],
) -> Callable[..., TemplateResponse]:
    """Render returned context in a table view."""

    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> TemplateResponse:
        request: HttpRequest = args[0]
        context = func(*args, **kwargs)
        func_name = f"{func.__module__}.{func.__qualname__}"
        urls: List[URLConfig] = admin_data_settings.URLS

        for item in urls:  # pylint: disable=too-many-nested-blocks
            view_name = f"{item['view'].__module__}.{item['view'].__qualname__}"
            if view_name == func_name:
                context["slug"] = item["route"]

                # Look for any item links in the rows.
                if item["items"] is not None:
                    item_view_name = item["items"]["name"]

                    for header in context["table"]:
                        for row_no, cell in enumerate(context["table"][header]):
                            if not isinstance(cell, ItemLink):  # pragma: no cover
                                continue

                            context["table"][header][row_no] = format_html(
                                '<a href="{}">{}</a>',
                                reverse(
                                    viewname=f"admin:{item_view_name}",
                                    kwargs=cell.kwargs,
                                    current_app=admin_data_settings.NAME,
                                ),
                                str(cell.link_item),
                            )

                        break  # Only the first column
                break  # Stop searching once view is found

        else:
            raise ValueError(f"Cannot find '{func_name}' in ADMIN_DATA_VIEWS setting.")

        # Transform the columns into a list of rows.
        # This way the table is easier to render in the template.
        context["headers"] = list(context["table"].keys())
        context["rows"]: List[List[Any]] = [[] for _ in next(iter(context["table"].values()))]
        for column in context["table"].values():
            for row_no, cell in enumerate(column):
                context["rows"][row_no].append(cell)

        context.update(admin.site.each_context(request))
        context["app_label"] = admin_data_settings.NAME
        request.current_app = admin_data_settings.NAME
        return TemplateResponse(request, "admin_data_views/admin_data_table_page.html", context)

    return wrapper


def render_with_item_view(
    func: Callable[..., ItemContext],
) -> Callable[..., TemplateResponse]:
    """Render returned context in an item view."""

    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> TemplateResponse:
        request: HttpRequest = args[0]
        context = func(*args, **kwargs)
        func_name = f"{func.__module__}.{func.__qualname__}"

        for item in admin_data_settings.URLS:
            # Allow item views separately
            view_name = f"{item['view'].__module__}.{item['view'].__qualname__}"
            if view_name == func_name:
                if context.get("slug") is None:
                    context["slug"] = item["route"]
                break

            # Item view inside table view definition
            if item["items"] is not None:
                view_name = f"{item['items']['view'].__module__}.{item['items']['view'].__qualname__}"
                if view_name == func_name:
                    if context.get("slug") is None:
                        context["slug"] = item["items"]["route"]
                    context["category_slug"] = item["route"]
                    context["category_url"] = reverse(
                        viewname=f"admin:{item['name']}",
                        current_app=admin_data_settings.NAME,
                    )
                    break
        else:
            raise ValueError(f"Cannot find '{func_name}' in ADMIN_DATA_VIEWS setting.")

        context.update(admin.site.each_context(request))
        context["app_label"] = admin_data_settings.NAME
        request.current_app = admin_data_settings.NAME
        return TemplateResponse(request, "admin_data_views/admin_data_item_page.html", context)

    return wrapper


class ItemLink:
    def __init__(self, link_item: Any, /, **kwargs: Any):
        """Marks a link to an item for the given view.

        :param link_item: Link text.
        :param args: Additional arguments to pass to the item route.
        """
        self.link_item = link_item
        self.kwargs = kwargs



from typing import Any, Dict, List, NamedTuple, Optional, Set, Union

from django.conf import settings
from django.test.signals import setting_changed
from settings_holder import holder, reload_settings

from .typing import URLConfig


__all__ = [
    "admin_data_settings",
]


class AdminDataViewsSettings(NamedTuple):
    #
    # URLs for top-level cateories. These will appear in the sidebar.
    # URLs for item pages. These will not appear in the sidebar.
    URLS: List[URLConfig] = []
    #
    # Name of the admin data views section in admin panel
    NAME: str = "Admin Data Views"


SETTING_NAME = "ADMIN_DATA_VIEWS"

USER_SETTINGS: Optional[Dict[str, Any]] = getattr(settings, SETTING_NAME, None)

DEFAULTS: Dict[str, Any] = AdminDataViewsSettings()._asdict()

IMPORT_STRINGS: Set[Union[bytes, str]] = {"URLS"}

REMOVED_SETTINGS: Set[str] = set()


class SettingsHolder(holder.SettingsHolder):
    def perform_import(self, val: str, setting: str) -> Any:
        if setting in {"URLS"}:
            val: List[URLConfig]
            for i, item in enumerate(val):
                missing = {"route", "view", "name"}.difference(item.keys())
                if missing:
                    raise RuntimeError(f"Missing keys in ADMIN_DATA_VIEWS[{i}]: {missing}")

                item["route"] = item["route"].rstrip("/").lstrip("/")
                item["view"] = self.import_from_string(item["view"], setting)

                if item.get("items") is None:
                    item["items"] = None
                    continue

                missing = {"route", "view", "name"}.difference(item["items"].keys())
                if missing:
                    raise RuntimeError(f"Missing keys in ADMIN_DATA_VIEWS[{i}]['items']: {missing}")

                item["items"]["route"] = item["items"]["route"].rstrip("/").lstrip("/")
                item["items"]["view"] = self.import_from_string(item["items"]["view"], setting)

            return val

        return super().perform_import(val, setting)  # pragma: no cover


admin_data_settings = SettingsHolder(
    user_settings=USER_SETTINGS,
    defaults=DEFAULTS,
    import_strings=IMPORT_STRINGS,
    removed_settings=REMOVED_SETTINGS,
)

reload_admin_data_settings = reload_settings(SETTING_NAME, admin_data_settings)
setting_changed.connect(reload_admin_data_settings)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['admin_data_views', 'admin_data_views.templatetags']

package_data = \
{'': ['*'], 'admin_data_views': ['templates/admin_data_views/*']}

install_requires = \
['Django>=3.2', 'django-settings-holder>=0.0.4']

setup_kwargs = {
    'name': 'django-admin-data-views',
    'version': '0.2.0',
    'description': 'Add custom data views to django admin panel.',
    'long_description': '# Django Admin Data Views\n\n[![Coverage Status][coverage-badge]][coverage]\n[![GitHub Workflow Status][status-badge]][status]\n[![PyPI][pypi-badge]][pypi]\n[![GitHub][licence-badge]][licence]\n[![GitHub Last Commit][repo-badge]][repo]\n[![GitHub Issues][issues-badge]][issues]\n[![Downloads][downloads-badge]][pypi]\n\n[![Python Version][version-badge]][pypi]\n[![Django Version][django-badge]][pypi]\n\n```shell\npip install django-admin-data-views\n```\n\n---\n\n**Documentation**: [https://mrthearman.github.io/django-admin-data-views/](https://mrthearman.github.io/django-admin-data-views/)\n\n**Source Code**: [https://github.com/MrThearMan/django-admin-data-views/](https://github.com/MrThearMan/django-admin-data-views/)\n\n---\n\nDjango Admin Data Views enables you to easily add non-model data to the django admin panel.\nData from an API or file can be shown in similar table and item views than regular models.\n\n\n![Example](docs/img/example.png)\n\n\n[coverage-badge]: https://coveralls.io/repos/github/MrThearMan/django-admin-data-views/badge.svg?branch=main\n[status-badge]: https://img.shields.io/github/workflow/status/MrThearMan/django-admin-data-views/Test\n[pypi-badge]: https://img.shields.io/pypi/v/django-admin-data-views\n[licence-badge]: https://img.shields.io/github/license/MrThearMan/django-admin-data-views\n[repo-badge]: https://img.shields.io/github/last-commit/MrThearMan/django-admin-data-views\n[issues-badge]: https://img.shields.io/github/issues-raw/MrThearMan/django-admin-data-views\n[version-badge]: https://img.shields.io/pypi/pyversions/django-admin-data-views\n[downloads-badge]: https://img.shields.io/pypi/dm/django-admin-data-views\n[django-badge]: https://img.shields.io/pypi/djversions/django-admin-data-views\n\n[coverage]: https://coveralls.io/github/MrThearMan/django-admin-data-views?branch=main\n[status]: https://github.com/MrThearMan/django-admin-data-views/actions/workflows/test.yml\n[pypi]: https://pypi.org/project/django-admin-data-views\n[licence]: https://github.com/MrThearMan/django-admin-data-views/blob/main/LICENSE\n[repo]: https://github.com/MrThearMan/django-admin-data-views/commits/main\n[issues]: https://github.com/MrThearMan/django-admin-data-views/issues\n',
    'author': 'Matti Lamppu',
    'author_email': 'lamppu.matti.akseli@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MrThearMan/django-admin-data-views',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)
